package jp.ossc.installer.tools.inventory;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import jp.ossc.installer.util.OpenStandiaXmlFile;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

import com.sun.org.apache.xml.internal.serializer.OutputPropertiesFactory;

/**
 * Cxg[fBNg[ɂ钊ۃNXłB<br>
 * @author S.Fujii
 * @version 1.0
 * @since 1.0
 */
public abstract class AbstaractInventory {

    /**
     * OpenStandia/Installerv_NgCXg[ĂȂꍇ̃o[W֕<br>
     */
    protected static final String NOT_INSTALLED_VALUE = "NOT_INSTALLED";

    /**
     * Cxg[i[XMLt@C̃ev[g̃t@CpX<br>
     */
    protected static final String INVENTORY_TEMPLATE_XML_FILE_PATH_NAME = "jp/ossc/installer/tools/inventory/openstandia-inventory-template.xml";

    /**
     * AvP[V擾ۂkey<br>
     */
    protected static final String STACK_APPLICATION_NAME_KEY = "stack.application.name";

    /**
     * AvP[Ṽo[W擾ۂkey<br>
     */
    protected static final String STACK_APPLICATION_VER_KEY = "stack.application.ver";

    /**
     * v_Ng擾ۂkey<br>
     */
    protected static final String PRODUCT_NAME = "product.name";

    /**
     * v_Ng̃o[W擾ۂkey<br>
     */
    protected static final String PRODUCT_VER = "product.ver";

    /**
     * G[bZ[W : t@Cǂݍ݂Ɏsꍇ<br>
     */
    protected static final String ERR_MSG_FAILED_READ_FILE = "t@Cǂݍ݂Ɏs܂B : [%s]";

    /**
     * xbZ[W : AvP[Vnull̏ꍇ<br>
     */
    protected static final String WARN_MSG_APPLICATION_NAME_IS_NULL = "AvP[VnullłB";

    /**
     * xbZ[W : AvP[Ṽo[Wnull̏ꍇ<br>
     */
    protected static final String WARN_MSG_APPLICATION_VERSION_IS_NULL = "AvP[Ṽo[WnullłB";

    /**
     * xbZ[W : v_Ngnull̏ꍇ<br>
     */
    protected static final String WARN_MSG_PRODUCT_NAME_IS_NULL = "v_NgnullłB";

    /**
     * xbZ[W : v_Ng̃o[Wnull̏ꍇ<br>
     */
    protected static final String WARN_MSG_PRODUCT_VERSION_IS_NULL = "v_Ng̃o[WnullłB";

    /**
     * X^bN`t@C<br>
     */
    protected Map<String, String> stackXmlMap;

    /**
     * OpenStandiaz[fBNg<br>
     */
    protected String openStandiaHomeDir;

    /**
     * Cxg[t@C<br>
     */
    protected File inventory;

    /**
     * Cxg[t@CɃCXg[L^c<br>
     * @param no v_NgNo
     * @param stackXml X^bN`t@C
     * @throws IOException IOException
     * @throws SAXException SAXException
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws TransformerException TransformerException
     */
    public void execute(final String no, final File stackXml)
        throws IOException, ParserConfigurationException, SAXException,
        TransformerException {

        // X^bN`t@Cǂݍ
        stackXmlMap = OpenStandiaXmlFile.getStackXmlProperties(stackXml);

        // OpenStandiaz[fBNg擾
        openStandiaHomeDir = OpenStandiaXmlFile
                .getOpenStandiaHome(stackXmlMap);

        // Cxg[t@CpX擾
        final String inventoryFilePath = OpenStandiaXmlFile
                .getInventoryFilePath(openStandiaHomeDir);

        // Cxg[t@C݂邩ǂ
        inventory = (new File(inventoryFilePath)).getAbsoluteFile();
        if (inventory.exists() && inventory.isFile()) {
            // ݂ꍇAȂ
        } else {
            // ݂ȂꍇAt@CVK쐬
            createInventoryFile();

            // ev[g`폜
            deleteApplicationNode("OpenStandia/Installer");
            deleteProductNode("name");
        }

        // AvP[Vyуo[Wݒ
        final String applicationName = stackXmlMap
                .get(STACK_APPLICATION_NAME_KEY);
        final String applicationVer = stackXmlMap
                .get(STACK_APPLICATION_VER_KEY);
        writeApplicationNode(applicationName, applicationVer);
    }

    /**
     * \[XɊ܂܂Ăev[gXMLt@C<br>
     * gpăCxg[t@CVKɐ<br>
     * @throws IOException IOException
     * @throws SAXException SAXException
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws TransformerException TransformerException
     */
    protected void createInventoryFile() throws IOException, SAXException,
        ParserConfigurationException, TransformerException {

        // ϐ錾
        OutputStream output = null;

        try {
            // `t@Cǂݍ
            final InputStream stream = getResouceInputStream(INVENTORY_TEMPLATE_XML_FILE_PATH_NAME);
            DocumentBuilderFactory dbfactory = DocumentBuilderFactory
                    .newInstance();
            DocumentBuilder builder = dbfactory.newDocumentBuilder();
            Document xtree = builder.parse(stream);

            // InventoryfBNg𐶐
            final File parentDirFile = inventory.getParentFile();
            parentDirFile.mkdirs();

            // ݐt@C𐶐
            output = new FileOutputStream(inventory);
            StreamResult result = new StreamResult(output);

            // `t@C̓e]
            DOMSource source = new DOMSource(xtree);
            TransformerFactory transFactory = TransformerFactory
                    .newInstance();
            Transformer transformer = transFactory.newTransformer();
            transformer.transform(source, result);
        } finally {
            try {
                output.close();
            } catch (Exception e) {
                // Ȃ
            }
        }
    }

    /**
     * Cxg[t@C폜<br>
     */
    protected void deleteInventoryFile() {
        if (inventory == null) {
            // Ȃ
        } else {
            // Cxg[t@C폜
            deleteFile(inventory);

            // Cxg[fBNgfBNgȂ폜
            final File parentDirFile = inventory.getParentFile();
            deleteDirectory(parentDirFile);
        }
    }

    /**
     * Cxg[t@CɉL<br>
     * EAvP[V<br>
     * EAvP[Ṽo[W<br>
     * @param name AvP[V
     * @param version AvP[Ṽo[W
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws SAXException SAXException
     * @throws IOException IOException
     * @throws TransformerException TransformerException
     */
    protected void writeApplicationNode(final String name,
            final String version) throws ParserConfigurationException,
        SAXException, IOException, TransformerException {
        if (name == null) {
            System.out.println(WARN_MSG_APPLICATION_NAME_IS_NULL);
        } else if (version == null) {
            System.out.println(WARN_MSG_APPLICATION_VERSION_IS_NULL);
        } else {
            // Cxg[t@Cǂݍ
            Document xtree = readInventoryFile(inventory);

            // stack^O擾
            final NodeList stackList = xtree.getElementsByTagName("stack");

            // applications^O擾
            final NodeList applicationsList = ((Element) stackList.item(0))
                    .getElementsByTagName("applications");

            // application^O擾
            final NodeList applicationList = ((Element) applicationsList
                    .item(0)).getElementsByTagName("application");

            // `ꂽv_NgJԂ
            boolean updateFlag = false;
            final int applicationLength = applicationList.getLength();
            for (int i = 0; i < applicationLength; i++) {

                // application̖O擾
                final Node applicationName = ((Element) applicationList
                        .item(i)).getElementsByTagName("name").item(0);
                final String applicationValue = applicationName
                        .getFirstChild().getNodeValue();

                // 擾v_NgƁAǉ悤ƂĂv_Ngr
                if (applicationValue.trim().equalsIgnoreCase(name.trim())) {
                    // ̏ꍇ́ACxg[t@CXV
                    // application̖Oݒ
                    applicationName.getFirstChild().setNodeValue(name);

                    // applicatioño[Wݒ
                    final Node applicationVer = ((Element) applicationList
                            .item(i)).getElementsByTagName("ver").item(0);
                    applicationVer.getFirstChild().setNodeValue(version);

                    // XVɐݒ
                    updateFlag = true;
                    break;
                } else {
                    // Ȃ
                }
            }

            // XVǂ
            if (updateFlag) {
                // XVꍇ́AȂ
            } else {
                // XVłȂꍇ́AVKǉ
                Element applicationElement = xtree
                        .createElement("application");

                // AvP[V
                Element nameElement = xtree.createElement("name");
                final Text nameText = xtree.createTextNode(name);
                nameElement.appendChild(nameText);

                // AvP[Ṽo[W
                Element verElement = xtree.createElement("ver");
                final Text verText = xtree.createTextNode(version);
                verElement.appendChild(verText);

                // applications^ủɒǉ
                applicationElement.appendChild(nameElement);
                applicationElement.appendChild(verElement);
                applicationsList.item(0).appendChild(applicationElement);
            }

            // XVe]
            writeInventoryFile(xtree, inventory);
        }

    }

    /**
     * Cxg[t@CɉL<br>
     * Ev_Ng<br>
     * Ev_Ng̃o[W<br>
     * @param name v_Ng
     * @param version v_Ng̃o[W
     * @throws IOException IOException
     * @throws SAXException SAXException
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws TransformerException TransformerException
     */
    protected void writeProductNode(final String name, final String version)
        throws ParserConfigurationException, SAXException, IOException,
        TransformerException {
        if (name == null) {
            System.out.println(WARN_MSG_PRODUCT_NAME_IS_NULL);
        } else if (version == null) {
            System.out.println(WARN_MSG_PRODUCT_VERSION_IS_NULL);
        } else {
            // Cxg[t@Cǂݍ
            Document xtree = readInventoryFile(inventory);

            // stack^O擾
            final NodeList stackList = xtree.getElementsByTagName("stack");

            // products^O擾
            final NodeList productsList = ((Element) stackList.item(0))
                    .getElementsByTagName("products");

            // product^O擾
            final NodeList productList = ((Element) productsList.item(0))
                    .getElementsByTagName("product");

            // `ꂽv_NgJԂ
            boolean updateFlag = false;
            final int productLength = productList.getLength();
            for (int i = 0; i < productLength; i++) {

                // product̖O擾
                final Node productName = ((Element) productList.item(i))
                        .getElementsByTagName("name").item(0);
                final String productValue = productName.getFirstChild()
                        .getNodeValue();

                // 擾v_NgƁAǉ悤ƂĂv_Ngr
                if (productValue.trim().equalsIgnoreCase(name.trim())) {
                    // ̏ꍇ́ACxg[t@CXV
                    // product̖Oݒ
                    productName.getFirstChild().setNodeValue(name);

                    // product̃o[Wݒ
                    final Node productVer = ((Element) productList.item(i))
                            .getElementsByTagName("ver").item(0);
                    productVer.getFirstChild().setNodeValue(version);

                    // XVɐݒ
                    updateFlag = true;
                    break;
                } else {
                    // Ȃ
                }
            }

            // XVǂ
            if (updateFlag) {
                // XVꍇ́AȂ
            } else {
                // XVłȂꍇ́AVKǉ
                Element productElement = xtree.createElement("product");

                // v_Ng
                Element nameElement = xtree.createElement("name");
                final Text nameText = xtree.createTextNode(name);
                nameElement.appendChild(nameText);

                // v_Ng̃o[W
                Element verElement = xtree.createElement("ver");
                final Text verText = xtree.createTextNode(version);
                verElement.appendChild(verText);

                // products^ủɒǉ
                productElement.appendChild(nameElement);
                productElement.appendChild(verElement);
                productsList.item(0).appendChild(productElement);
            }

            // XVe]
            writeInventoryFile(xtree, inventory);
        }
    }

    /**
     * Cxg[t@CAvP[V폜<br>
     * @param name 폜AvP[V
     * @throws IOException IOException
     * @throws SAXException SAXException
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws TransformerException TransformerException
     */
    protected void deleteApplicationNode(final String name)
        throws ParserConfigurationException, SAXException, IOException,
        TransformerException {
        if (name == null) {
            System.out.println(WARN_MSG_APPLICATION_NAME_IS_NULL);
        } else {
            // Cxg[t@Cǂݍ
            Document xtree = readInventoryFile(inventory);

            // stack^O擾
            final NodeList stackList = xtree.getElementsByTagName("stack");

            // applications^O擾
            final NodeList applicationsList = ((Element) stackList.item(0))
                    .getElementsByTagName("applications");

            // application^O擾
            final NodeList applicationList = ((Element) applicationsList
                    .item(0)).getElementsByTagName("application");

            // `ꂽAvP[VJԂ
            final int applicationLength = applicationList.getLength();
            for (int i = 0; i < applicationLength; i++) {

                // application̖O擾
                final Node applicationName = ((Element) applicationList
                        .item(i)).getElementsByTagName("name").item(0);
                final String applicationValue = applicationName
                        .getFirstChild().getNodeValue();

                // 擾AvP[VƁA폜悤ƂĂAvP[Vr
                if (applicationValue.trim().equalsIgnoreCase(name.trim())) {
                    // ̏ꍇ́ACxg[t@C폜
                    final Node parentNode = applicationList.item(i)
                            .getParentNode();
                    parentNode.removeChild(applicationList.item(i));
                    break;
                } else {
                    // Ȃ
                }
            }

            // XVe]
            writeInventoryFile(xtree, inventory);
        }
    }

    /**
     * Cxg[t@Cv_Ng폜<br>
     * @param name 폜v_Ng
     * @throws IOException IOException
     * @throws SAXException SAXException
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws TransformerException TransformerException
     */
    protected void deleteProductNode(final String name)
        throws ParserConfigurationException, SAXException, IOException,
        TransformerException {
        if (name == null) {
            System.out.println(WARN_MSG_PRODUCT_NAME_IS_NULL);
        } else {
            // Cxg[t@Cǂݍ
            Document xtree = readInventoryFile(inventory);

            // stack^O擾
            final NodeList stackList = xtree.getElementsByTagName("stack");

            // products^O擾
            final NodeList productsList = ((Element) stackList.item(0))
                    .getElementsByTagName("products");

            // product^O擾
            final NodeList productList = ((Element) productsList.item(0))
                    .getElementsByTagName("product");

            // `ꂽv_NgJԂ
            final int productLength = productList.getLength();
            for (int i = 0; i < productLength; i++) {

                // product̖O擾
                final Node productName = ((Element) productList.item(i))
                        .getElementsByTagName("name").item(0);
                final String productValue = productName.getFirstChild()
                        .getNodeValue();

                // 擾v_NgƁA폜悤ƂĂv_Ngr
                if (productValue.trim().equalsIgnoreCase(name.trim())) {
                    // ̏ꍇ́ACxg[t@C폜
                    final Node parentNode = productList.item(i)
                            .getParentNode();
                    parentNode.removeChild(productList.item(i));
                    break;
                } else {
                    // Ȃ
                }
            }

            // XVe]
            writeInventoryFile(xtree, inventory);
        }
    }

    /**
     * Cxg[t@Cv_Ng񂪑݂邩ǂ<br>
     * @param name 擾v_Ng
     * @return true : ݂ / false : ݂Ȃ
     * @throws IOException IOException
     * @throws SAXException SAXException
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws TransformerException TransformerException
     */
    protected boolean existsProductNode(final String name)
        throws ParserConfigurationException, SAXException, IOException,
        TransformerException {
        final String ver = getProductVerNode(name);
        if (ver.equals(NOT_INSTALLED_VALUE)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Cxg[t@CAvP[Ṽo[W擾<br>
     * @param name 擾AvP[V
     * @return AvP[Ṽo[W
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws SAXException SAXException
     * @throws IOException IOException
     */
    protected String getApplicationVerNode(final String name)
        throws ParserConfigurationException, SAXException, IOException {
        String ver = NOT_INSTALLED_VALUE;
        if (inventory == null || name == null) {
            // Ȃ
        } else {
            // Cxg[t@Cǂݍ
            Document xtree = readInventoryFile(inventory);

            // stack^O擾
            final NodeList stackList = xtree.getElementsByTagName("stack");

            // applications^O擾
            final NodeList applicationsList = ((Element) stackList.item(0))
                    .getElementsByTagName("applications");

            // application^O擾
            final NodeList applicationList = ((Element) applicationsList
                    .item(0)).getElementsByTagName("application");

            // `ꂽAvP[VJԂ
            final int applicationLength = applicationList.getLength();
            for (int i = 0; i < applicationLength; i++) {

                // application̖O擾
                final Node applicationName = ((Element) applicationList
                        .item(i)).getElementsByTagName("name").item(0);
                final String applicationValue = applicationName
                        .getFirstChild().getNodeValue();

                // 擾AvP[VƁA擾AvP[Vr
                if (applicationValue.trim().equalsIgnoreCase(name.trim())) {
                    // ̏ꍇ
                    // applicatioño[W擾
                    final Node verName = ((Element) applicationList.item(i))
                            .getElementsByTagName("ver").item(0);
                    final String verValue = verName.getFirstChild()
                            .getNodeValue();
                    ver = verValue;
                    break;
                } else {
                    // Ȃ
                }
            }
        }

        // Ԃl̐ݒ
        return ver;
    }

    /**
     * Cxg[t@Cw肳ꂽv_Ng̃o[W擾<br>
     * @param name 擾v_Ng
     * @return v_Ng̃o[W
     * @throws IOException IOException
     * @throws SAXException SAXException
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws TransformerException TransformerException
     */
    protected String getProductVerNode(final String name)
        throws ParserConfigurationException, SAXException, IOException,
        TransformerException {

        String ver = NOT_INSTALLED_VALUE;
        if (name == null) {
            // Ȃ
        } else {
            // Cxg[t@Cǂݍ
            Document xtree = readInventoryFile(inventory);

            // stack^O擾
            final NodeList stackList = xtree.getElementsByTagName("stack");

            // products^O擾
            final NodeList productsList = ((Element) stackList.item(0))
                    .getElementsByTagName("products");

            // product^O擾
            final NodeList productList = ((Element) productsList.item(0))
                    .getElementsByTagName("product");

            // `ꂽv_NgJԂ
            final int productLength = productList.getLength();
            for (int i = 0; i < productLength; i++) {

                // product̖O擾
                final Node productName = ((Element) productList.item(i))
                        .getElementsByTagName("name").item(0);
                final String productValue = productName.getFirstChild()
                        .getNodeValue();

                // 擾v_NgƁA擾v_Ngr
                if (productValue.trim().equalsIgnoreCase(name.trim())) {
                    // ̏ꍇ
                    // product̃o[W擾
                    final Node verName = ((Element) productList.item(i))
                            .getElementsByTagName("ver").item(0);
                    final String verValue = verName.getFirstChild()
                            .getNodeValue();
                    ver = verValue;
                    break;
                } else {
                    // Ȃ
                }
            }
        }

        // Ԃl̐ݒ
        return ver;
    }

    /**
     * Cxg[t@CCXg[ς݂̃v_Ng擾<br>
     * @return v_Ng
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws SAXException SAXException
     * @throws IOException IOException
     */
    protected int getProductNodeNum() throws ParserConfigurationException,
        SAXException, IOException {
        // Cxg[t@Cǂݍ
        Document xtree = readInventoryFile(inventory);

        // stack^O擾
        final NodeList stackList = xtree.getElementsByTagName("stack");

        // products^O擾
        final NodeList productsList = ((Element) stackList.item(0))
                .getElementsByTagName("products");

        // product^O擾
        final NodeList productList = ((Element) productsList.item(0))
                .getElementsByTagName("product");

        // Ԃl̐ݒ
        return productList.getLength();
    }

    /**
     * Cxg[t@Cǂݍ<br>
     * @param inventory Cxg[t@C
     * @return Cxg[t@C͂
     * @throws ParserConfigurationException ParserConfigurationException
     * @throws SAXException SAXException
     * @throws IOException IOException
     */
    protected Document readInventoryFile(final File inventory)
        throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory dbfactory = DocumentBuilderFactory
                .newInstance();
        DocumentBuilder builder = dbfactory.newDocumentBuilder();
        final Document xtree = builder.parse(inventory);
        return xtree;
    }

    /**
     * Cxg[t@Cɓe]<br>
     * @param xtree ݓeDocument
     * @param outputFile ݐt@C
     * @throws FileNotFoundException FileNotFoundException
     * @throws TransformerException TransformerException
     */
    protected void writeInventoryFile(final Document xtree,
            final File outputFile) throws FileNotFoundException,
        TransformerException {
        final DOMSource source = new DOMSource(xtree);
        // ϐ錾
        OutputStream output = null;
        try {
            // ݐt@C𐶐
            output = new FileOutputStream(outputFile);
            Result result = new StreamResult(output);
            TransformerFactory transFactory = TransformerFactory
                    .newInstance();
            Transformer transformer = transFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty(
                javax.xml.transform.OutputKeys.METHOD, "xml");
            transformer.setOutputProperty(
                OutputPropertiesFactory.S_KEY_INDENT_AMOUNT, "4");
            transformer.transform(source, result);
        } finally {
            try {
                output.close();
            } catch (Exception e) {
                // Ȃ
            }
        }
    }

    /**
     * w肳ꂽ΃pX烊\[XpX擾<br>
     * @param filePath ΃pX
     * @return \[XpX
     * @throws IOException IOException
     */
    protected String getResoucePath(final String filePath) throws IOException {
        // NX[_[̎擾
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        if (cl == null) {
            cl = this.getClass().getClassLoader();
        } else {
            // Ȃ
        }
        if (cl == null) {
            throw new IOException(String.format(
                ERR_MSG_FAILED_READ_FILE, filePath));
        } else {
            // Ȃ
        }

        // ΃pX̎擾
        final URL url = cl.getResource(filePath);
        if (url == null) {
            throw new IOException(String.format(
                ERR_MSG_FAILED_READ_FILE, filePath));
        } else {
            // Ȃ
        }
        final String resourcePath = url.getPath();
        return resourcePath;
    }

    /**
     * w肳ꂽ΃pX烊\[XInputStream擾<br>
     * @param filePath ΃pX
     * @return \[XInputStream
     * @throws IOException IOException
     */
    protected InputStream getResouceInputStream(final String filePath)
        throws IOException {
        // NX[_[̎擾
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        if (cl == null) {
            cl = this.getClass().getClassLoader();
        } else {
            // Ȃ
        }
        if (cl == null) {
            throw new IOException(String.format(
                ERR_MSG_FAILED_READ_FILE, filePath));
        } else {
            // Ȃ
        }

        // ΃pX̎擾
        final InputStream stream = cl.getResourceAsStream(filePath);
        if (stream == null) {
            throw new IOException(String.format(
                ERR_MSG_FAILED_READ_FILE, filePath));
        } else {
            // Ȃ
        }
        return stream;
    }

    /**
     * Ώۂ̃t@C폜<br>
     * @param target 폜t@C
     */
    private void deleteFile(final File target) {
        if (target == null || !target.isFile()) {
            // Ȃ
        } else {
            boolean flag = false;
            try {
                flag = target.delete();
            } catch (Exception e) {
                // Ȃ
            }
            if (!flag) {
                try {
                    target.deleteOnExit();
                } catch (Exception e) {
                    // Ȃ
                }
            } else {
                // Ȃ
            }
        }
    }

    /**
     * Ώۂ̃fBNg폜<br>
     * @param target 폜fBNg
     */
    private void deleteDirectory(final File target) {
        if (target == null || !target.isDirectory()) {
            // Ȃ
        } else {
            boolean flag = false;
            final String[] filesInTargetDir = target.list();
            if (filesInTargetDir == null || filesInTargetDir.length == 0) {
                try {
                    flag = target.delete();
                } catch (Exception e) {
                    // Ȃ
                }
                if (!flag) {
                    try {
                        target.deleteOnExit();
                    } catch (Exception e) {
                        // Ȃ
                    }
                } else {
                    // Ȃ
                }
            } else {
                // Ȃ
            }
        }

    }

}
